# --- [ net_io.py ] ------------------------------------------------------------
#
# Demo für AVR-NET-IO in Python 3.2 ohne graphische Oberfläche
#
# liest die ADC-Werte, den Status der 8 Ausgänge und 4 Eingänge
# toggelt nach jedem Messintervall einen Ausgang
# bedient das LCD mit den Messwerten - sofern ein LCD angeschlossen ist.
#
# 01.11.12 
#
# -------------------------------------------------------------------------------

import time
import telnetlib


telnet_host    = '192.168.0.90'
telnet_port    = 50290
telnet_timeout = 0.5
log_interval   = 2             # Messintervall in Sekunden
LCD_OK         = False

# ------------------------------------------------------------------------------
# Ein Kommando senden und die Antwort zurückgeben
# Das Ende der Antwort wird am Linefeed "\n" erkannt

def Send_Cmd(out, cmd):
    out.write(cmd.encode('ascii') + b"\n")
    return out.read_until(b'\n', telnet_timeout)


# ------------------------------------------------------------------------------
# Die Software-Version holen und als Liste zeilenweise zurückgeben
# Die Information besteht aus vier Zeilen, die jeweils mit \r\n abgeschlossen sind
# Daher kann das Ende des Textes nicht anhand \n erkannt werden
# Die 4.Zeile ist immer leer.

def Get_Version(out):
    out.write("VERSION".encode('ascii') + b"\n")
    time.sleep(0.2)             # warten bis die Daten bereitstehen
    t = (out.read_eager()).decode('ascii')

    return t.split("\r\n")        # in Zeilen aufspalten

# ------------------------------------------------------------------------------
# Einen einzelnen ADC-Wert auslesen
# Bei einem Fehler wird -1 zurückgeliefert

def Get_Adc(out, channel):
    try:
        adc_wert = int(Send_Cmd(out, "GETADC " + str(channel)))
    except:
        adc_wert = -1
    return adc_wert

# ------------------------------------------------------------------------------
# Alle vier ADC-Werte auslesen und als Liste von Int's zurückliefern

def Get_Adcs(out):
    l = [0,0,0,0]

    for i in [1,2,3,4]:
        l[i-1] = Get_Adc(out, i)
        
    return l

# ------------------------------------------------------------------------------
# holte den aktuellen Status der AusgangsPins

def Get_Port_State(out):
    l = [0,0,0,0,0,0,0,0]
    t = (Send_Cmd(out, "GETSTATUS")).decode('ascii')
    t = t[1:10]                     # führendes "S" und folgendes "\r\n" abschneiden
    for i in range(8):
        l[i] = t[7-i]               # Reihenfolge umkehren
        
    return l
# ------------------------------------------------------------------------------
# setzt einen der AusgangsPins

def Set_Port_State(out, pin, state):
    Send_Cmd(out, "SETPORT " + str(pin) + '.' + str(state))
    
# ------------------------------------------------------------------------------
# holte den aktuellen Status der Eingänge

def Get_Pin_State(out):
    l = [0,0,0,0]
    for i in [1,2,3,4]:
        t = (Send_Cmd(out, "GETPORT " + str(i))).decode('ascii')
        l[i-1] = t[0]               # folgendes \r\n' abschneiden
    return l

# --- LCD ----------------------------------------------------------------------
# ------------------------------------------------------------------------------

def Init_LCD(out):
    ans = (Send_Cmd(out,"INITLCD")).decode('ascii')
    return (ans[:3] == "ACK")

# -------------------------------------------------------------------------------    
# Schreibt eine Zeile, Zeile [1,2], maximale Zeichenzahl 16
# kürzer Zeichenfolgen werden mit Leerzeichen auf Länge 16 erweitert:
# dadurch wird das Zeilenende automatisch gelöscht.

def Write_LCD(out, zeile, txt):

    if len(txt) < 16:               # wenn weniger als 16 Zeichen, 
        txt.ljust(16)               # dann rechts mit ' ' auffüllen
    elif len(txt) > 16:             # wenn länger als 16 Zeichen
        txt = txt[0:16]             # dann auf 16 Zeichen kappen
        
    ans = (Send_Cmd(out, "WRITELCD " + str(zeile) + "." + txt)).decode('ascii')
    return(ans[:3] == "ACK")

# ------------------------------------------------------------------------------
# löscht eine Zeile (wenn der 2. Paramter angegeben wird als Zeile[1,2])
# ohne 2. Parameter wird das gesamte Display gelöscht

def Clear_LCD(out, zeile = 0):
    if zeile == 0:
        ans = (Send_Cmd(out, "CLEARLCD")).decode('ascii')
    else:
        ans = (Send_Cmd(out, "CLEARLCD " + str(zeile))).decode('ascii')

    return(ans[:3] == "ACK")


# -----------------------------------------------------------------------------
def net_io_init():
    global LCD_OK
    try:
        avrnetio = telnetlib.Telnet(telnet_host, telnet_port, telnet_timeout)

        for i in Get_Version(avrnetio):
            print(i)

        LCD_OK = Init_LCD(avrnetio)    
        avrnetio.close()
    
    except:
        print("Error Host:", telnet_host, "Port:", telnet_port)
        exit()

    return avrnetio


# ------------------------------------------------------------------------------    
# ------------------------------------------------------------------------------

if __name__ == '__main__':

    print("Start")
    print()

    netio = net_io_init()
    
    if LCD_OK:
        print("LCD initialisiert")
    else:
        print("kein LCD gefunden")
    print()

    pin   = 1
    state = 1

    while True:           
        print(time.ctime()[11:-5], end = ' ')
    
        netio.open(telnet_host, telnet_port)

        adc = Get_Adcs(netio)
        for i in adc:
            print(i, end = ' ')

        output = Get_Port_State(netio)

        zeile = (str(output[7]) + str(output[6]) +
                 str(output[5]) + str(output[4]) +
                 str(output[3]) + str(output[2]) +
                 str(output[1]) + str(output[0]))

        print(zeile, end = ' ')

        input = Get_Pin_State(netio)
        for i in input:
            print(i, end = ' ')

        print()
        Set_Port_State(netio, pin, state)    
        if LCD_OK:
            Clear_LCD(netio)
            zeile1 = (str(adc[0]).rjust(4) + str(adc[1]).rjust(4) +
                      str(adc[2]).rjust(4) + str(adc[3]).rjust(4))
            Write_LCD(netio, 1, zeile1)
            

            zeile2 = (str(input[0]) + str(input[1]) + str(input[2]) +
                      str(input[3]) + " : " + zeile)

            Write_LCD(netio, 2, zeile2)

        netio.close()
    
        time.sleep(log_interval)
        pin += 1

        if pin > 8:
            pin = 1
            if state:
                state = 0
            else:
                state = 1



# --- [ eof ] ------------------------------------------------------------------

    

    
